/**************************************************************************/
/**	File:  		GWRPS.H										                   **/
/**	Author:  	Terry W. Jackson													 **/
/**	Company:		U. S. Nuclear Regulatory Comm.								 **/
/**	Date:			February 28, 1997													 **/
/**----------------------------------------------------------------------**/
/**	Description:  This is the header file for the Generic Westinghouse **/
/**     Reactor Protection System Simulation Program.                    **/
/**************************************************************************/

/*************************** Define Statements ****************************/
#define TRUE 1
#define FALSE 0
#define USER_SIZE 21


/****************************** Structures ********************************/
struct Status
{
	unsigned Key             : 1;			// 1 - continue(def.), 0 - exit
	unsigned Update_Screen_1 : 1;			// 1st screen update, default true
	unsigned Update_Screen_2 : 1;			// 2nd screen update, default true
	unsigned Update_Screen_3 : 1;			// 3rd screen flag, default off
	unsigned Update_Screen_4 : 1;			// 4th screen flag, default off
	unsigned RESET           : 1;			// TRUE - reset simulation
	unsigned TRIP            : 1;			// TRUE - enable simulation
	unsigned Screen_Select	 : 4;			// Current screen, default 1st
	unsigned SIMULATE			 : 5;			// Controls simulated variables
};

typedef struct SRNF
{
	double mantissa;
	int exponent;
}SRNF_Type;

typedef struct message
{
	char line_1[21];
	char line_2[21];
	char line_3[21];
	char line_4[21];
} message_type;

typedef char c[];


